;; -*- lexical-binding: t; -*-

;;;;; Shift+Insert pastes from primary selection, like xterm.

(defun elmord-primary-selection-yank ()
  "Yanks text from the primary selection (the text you just marked with
the mouse in a graphical environment), as opposed to the clipboard (the
place things end up in when you copy with Ctrl+C)."
  (interactive)
  (let ((x-select-enable-clipboard nil)
        (x-select-enable-primary t))
    (yank)))

(global-set-key (kbd "S-<insert>") 'elmord-primary-selection-yank)

;;;;; Insert current date.

(defun elmord-insert-current-date ()
  (interactive)
  (insert (format-time-string "%Y-%m-%d")))

(defun elmord-insert-current-date-and-time ()
  (interactive)
  (insert (format-time-string "%Y-%m-%d %H:%M %z")))

(define-key ctl-z-map (kbd "d") 'elmord-insert-current-date)
(define-key ctl-z-map (kbd "D") 'elmord-insert-current-date-and-time)


;;;;; C-u kills line backward (like in terminal/Vim).

(defvar elmord-backward-kill-line-preserve-indentation t
  "Whether `backward-kill-line' should avoid deleting indentation.")

(defun elmord-backward-kill-line (&optional prefix)
  "Kill from point to the beginning of the line.

If `elmord-backward-kill-line-preserve-indentation' is non-nil,
only kill up to indentation, unless point is already at
indentation, in which case kill to the beginning of the line.

With a prefix N, just calls (`kill-line' -N)."
  (interactive "P")
  (if prefix
      (kill-line (- prefix))
    (if (bolp)
        ;; What to do if we are at the beginning of line?
        ;; Currently, delete the previous newline, like Vim.
        (backward-delete-char 1 t)
      (if elmord-backward-kill-line-preserve-indentation
          (elmord-backward-kill-line-preserving-indentation)
        (kill-line 0)))))

(defun elmord-backward-kill-line-preserving-indentation ()
  (let ((indentation (save-excursion (back-to-indentation) (point))))
    (if (<= (point) indentation)
        (kill-line 0)
      (kill-region (point) indentation))))

(global-set-key "\C-u" 'elmord-backward-kill-line)

;;;;; Move universal argument to C-v (daring proposition!).
(global-set-key "\C-v" 'universal-argument)
(define-key universal-argument-map "\C-u" nil)
(define-key universal-argument-map "\C-v" 'universal-argument-more)

;;;;; Other edit commands.
(global-set-key (kbd "C-M-<backspace>") 'backward-kill-sexp)

(global-set-key (kbd "M-p") 'previous-error)
(global-set-key (kbd "M-n") 'next-error)

(defun elmord-backward-kill-char ()
  (interactive)
  (backward-delete-char-untabify 1 t))

(global-set-key (kbd "S-<backspace>") 'elmord-backward-kill-char)


(defun elmord-new-line-after ()
  (interactive)
  (end-of-line)
  (newline nil t))

(defun elmord-new-line-before ()
  (interactive)
  (previous-line)
  (elmord-new-line-after))

(global-set-key "\C-o" 'elmord-new-line-after)
(global-set-key "\M-o" 'elmord-new-line-before)


;; https://stackoverflow.com/questions/2471557/how-to-undo-fill-paragraph-in-emacs

(defun elmord-unfill-paragraph ()
  (interactive)
  (let ((fill-column (point-max)))
    (fill-paragraph nil)))

(defun elmord-unfill-region ()
  (interactive)
  (let ((fill-column (point-max)))
    (fill-region (region-beginning) (region-end) nil)))
