;; -*- lexical-binding: t; -*-

;;(global-set-key (kbd "C-f") 'find-file)
;;(global-set-key (kbd "C-b") 'ido-switch-buffer)
(global-set-key (kbd "C-o") 'other-window)
(global-set-key (kbd "C-S-o") (lambda () (interactive) (other-window -1)))

(global-set-key (kbd "C-1") 'delete-other-windows)
(global-set-key (kbd "C-2") 'split-window-below)
(global-set-key (kbd "C-3") 'split-window-right)
(global-set-key (kbd "C-0") 'delete-window)

(global-set-key (kbd "C-5") 'make-frame-command)

;;;;; Kill all unmodified buffers.

(defun elmord-exwm-buffer-p (buffer)
  (eq (buffer-local-value 'major-mode buffer) 'exwm-mode))

(defun elmord-kill-unmodified-buffers ()
  "Kill all unmodified buffers. Ignores EXWM buffers."
  (interactive)
  (let ((killed nil))
    (dolist (buffer (buffer-list))
      (when (not (or (buffer-modified-p buffer)
                     (equal (buffer-name buffer) "*scratch*")
                     (elmord-exwm-buffer-p buffer)))
        (push (buffer-name buffer) killed)
        (kill-buffer buffer)))
    (message "%d buffers killed: %s" (length killed) killed)))

(global-set-key (kbd "C-x K") 'elmord-kill-unmodified-buffers)


;;;;; ESC to quit temporary windows.

;; Keep track of which buffers are temporary.
(add-hook 'temp-buffer-window-setup-hook
          (defun elmord-mark-buffer-as-temporary ()
            (setq-local elmord-buffer-temporary-p t)))

(defvar elmord-temporary-buffer-names
  '("*Messages*" "*Completions*" "*Occur*")
  "Names of buffers which are always considered temporary, regardless of
the value of their value of `buffer-temporary-p'.")

(defun elmord-window-temporary-p (window)
  "Returns whether WINDOW is associated with a temporary buffer."
  (with-current-buffer (window-buffer window)
    (or (and (boundp 'elmord-buffer-temporary-p) elmord-buffer-temporary-p)
        (member (buffer-name) elmord-temporary-buffer-names))))

(defun elmord-quit-temp-windows-or-delete-last-window ()
  "Quit all temporary buffers in the current frame. If no such buffer exists,
delete the last window in the current frame (defined by the order of
`window-list'), excluding the current one. Returns t if a window was
quitted/deleted, nil otherwise."
  (interactive)
  (or (elmord-quit-temp-windows)
      (elmord-delete-last-window)
      (message "No window to delete")))

(defun elmord-quit-temp-windows ()
  "Quit all temporary buffers in the current frame.
 Returns t is any window was quitted, nil otherwise."
  (interactive)
  (let ((some-quitted-p nil))
    (dolist (window (window-list))
      (when (elmord-window-temporary-p window)
        (quit-window nil window)
        (setq some-quitted-p t)))
    some-quitted-p))

(defun elmord-delete-last-window ()
  "Delete the last window in the current frame (defined by the order of
`window-list', excluding the current window. Returns t if a window was
deleted, nil otherwise."
  (interactive)
  (let* ((windows (remove (get-buffer-window) (window-list)))
         (victim (car (last windows))))
    (when victim
      (delete-window victim)
      t)))

(global-set-key (kbd "<escape>") 'elmord-quit-temp-windows-or-delete-last-window)

;; In minibuffer, ESC behaves the same as C-g.
(define-key minibuffer-local-map (kbd "<escape>") 'abort-recursive-edit)

;; Changing window focus.
(global-set-key (kbd "M-S-<up>") 'windmove-up)
(global-set-key (kbd "M-S-<down>") 'windmove-down)
(global-set-key (kbd "M-S-<left>") 'windmove-left)
(global-set-key (kbd "M-S-<right>") 'windmove-right)

;; Breaks arrows in terminals :(
;;(global-set-key (kbd "M-O") 'other-window)

;; C-x b should offer the last active buffer as the default option,
;; even if it is visible in another window.
(defadvice other-buffer (around allow-visible-buffers)
  (ad-set-arg 1 t)
  ad-do-it)

(ad-activate 'other-buffer)

;;;; Completion window behavior.

;;;;; Do not auto-size completion windows.

(add-to-list 'display-buffer-alist
             '("\\*Completions\\*"
               (display-buffer-reuse-window
                display-buffer-pop-up-window
                display-buffer-use-some-window)
               (inhibit-same-window . t)
               (window-height . ignore) )
         )

(defadvice minibuffer-completion-help (around elmord-do-not-fit-window activate)
  (cl-letf (((symbol-function 'fit-window-to-buffer) #'ignore))
    ad-do-it))

(defadvice completion-at-point (around elmord-do-not-fit-window activate)
  (cl-letf (((symbol-function 'fit-window-to-buffer) #'ignore))
    ad-do-it))



(defun toggle-frame-fullscreen* ()
  (interactive)
  (toggle-frame-fullscreen)
  (if (eq (cdr (assoc 'fullscreen (frame-parameters))) 'fullboth)
      (display-time-mode 1)
    (display-time-mode 0)))

(global-set-key (kbd "<f11>") 'toggle-frame-fullscreen*)
