;; -*- lexical-binding: t; -*-

;;;; Programming modes.

;;;;; Tabs and spaces.

;; Use spaces instead of tabs.
(setq-default indent-tabs-mode nil)
(setq tab-always-indent 'complete)

;; Cleanup trailing whitespace.
(add-hook 'prog-mode-hook
  (defun elmord-prog-mode-hook ()
    (add-hook 'before-save-hook 'whitespace-cleanup nil t)))

(add-hook 'sgml-mode-hook
  (defun elmord-sgml-mode-hook ()
    (add-hook 'before-save-hook 'whitespace-cleanup nil t)))


;;;;; Compilation.

(defun elmord-compile-prompt-only-once (&optional read-command-p)
  "Invoke `compile' on the current buffer. Only ask for the compilation command
the first time it is called, or if a prefix argument is used."
  (interactive "P")
  (when read-command-p
    (setq-local compilation-read-command t))
  (call-interactively 'compile)
  (setq-local compilation-read-command nil))

(global-set-key (kbd "C-z C-z") 'elmord-compile-prompt-only-once)

(setq compilation-scroll-output t)
(setq compilation-ask-about-save nil)


(defun elmord-git-commit-and-push (message)
  (interactive "sCommit message: ")
  (save-some-buffers (not compilation-ask-about-save)
                     compilation-save-buffers-predicate)
  (shell-command (concat "git commit -a -m "
                         (shell-quote-argument message)
                         " && git push")))

(global-set-key (kbd "C-z m") 'elmord-git-commit-and-push)


;; C indentation.

(defvar elmord-c-style
  '("k&r"  ;; Inherit from this style
    (c-basic-offset . 4)
    (c-offsets-alist
     (case-label . 4)
     (statement-case-intro . 4)
     (statement-case-open . 4)
     (label . 2))))

(c-add-style "elmord" elmord-c-style)
(setq c-default-style "elmord")
