;; -*- lexical-binding: t; -*-

(setq comint-input-ring-size 10000)

;; Do I even use most of the things here?

;;;;;; Shell/Term-mode alternation (http://emacswiki.org/emacs/ShellMode)

;; (require 'shell)
;; (require 'term)

(defun elmord-term-switch-to-shell-mode ()
  (interactive)
  (if (equal major-mode 'term-mode)
      (progn
        (shell-mode)
        (set-process-filter  (get-buffer-process (current-buffer)) 'comint-output-filter )
        ;;(local-set-key (kbd "C-j") 'term-switch-to-shell-mode)
        (compilation-shell-minor-mode 1)
        (comint-send-input)
        )
    (progn
      (compilation-shell-minor-mode -1)
      (font-lock-mode -1)
      (set-process-filter  (get-buffer-process (current-buffer)) 'term-emulate-terminal)
      (term-mode)
      (term-char-mode)
      (term-send-raw-string (kbd "C-l"))
      )))

(add-hook 'shell-mode-hook
  (defun elmord-shell-mode-hook ()
    ;; By default, Emacs ignores files with some extensions when completing filenames
    ;; (mainly object files such as .o). This should not apply to shell completion.
    (make-local-variable 'completion-ignored-extensions)
    (setq completion-ignored-extensions nil)
    ;; Fix process window size (SIGWINCH) when it changes.
    (add-hook 'window-configuration-change-hook 'elmord-comint-fix-window-size)
    (compilation-shell-minor-mode)))

;; https://stackoverflow.com/questions/7987494/emacs-shell-mode-display-is-too-wide-after-splitting-window/11255996#11255996
;; Adapted because Emacs kept calling it after the process died.
(defun elmord-comint-fix-window-size ()
  "Change process window size."
  (when (derived-mode-p 'comint-mode)
    (let ((process (get-buffer-process (current-buffer))))
      (when process
        (set-process-window-size process
                                 (window-height)
                                 (window-width))))))

;; Don't create a new window for M-x shell, just like Emacs 24
;; (code from Emacs 25.1 NEWS file).
(add-to-list 'display-buffer-alist
             '("^\\*shell\\*" . (display-buffer-same-window)))

(eval-after-load 'comint
  (defun elmord-comint-eval-after-load ()
    (define-key comint-mode-map (kbd "C-r") 'comint-history-isearch-backward-regexp)
    (define-key comint-mode-map (kbd "C-p") 'comint-previous-input)
    (define-key comint-mode-map (kbd "C-n") 'comint-next-input)
    (add-hook 'comint-output-filter-functions 'force-mode-line-update)
    ))

;; Make prompt read-only.
(defun elmord-comint-backspace-no-delete-prompt (n)
  (interactive "p")
  (cond ((and (> (point) 1)
              (elmord-is-or-contains
               'comint-highlight-prompt
               (get-char-property (1- (point)) 'font-lock-face (current-buffer))))
         (message "Not stepping over prompt"))
        (t (delete-backward-char n))))

(defun elmord-is-or-contains (item list)
  (or (eq item list)
      (member item list)))

(eval-after-load 'shell
  (defun elmord-shell-eval-after-load ()
    (define-key shell-mode-map (kbd "DEL") 'elmord-comint-backspace-no-delete-prompt)
    (setq shell-dynamic-complete-functions
          (remove 'pcomplete-completions-at-point shell-dynamic-complete-functions))))
